<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

use sly\Assets\Util;

abstract class WV_Sally {
	const CLANG_CURRENT = -1; ///< int  Pseudo-Sprache für die aktuelle Sprache
	const CLANG_ALL     = -2; ///< int  Pseudo-Sprache für "jede Sprache"

	public static function init($controller, array $params) {
		// init main CSS and JS

		$container = $params['container'];
		$layout    = $container['sly-layout'];
		$name      = $params['name'];

		$layout->addCSSFile(Util::addOnUri('webvariants/developer-utils', 'css/backend.less'));
		$layout->addJavaScriptFile(Util::addOnUri('webvariants/developer-utils', 'js/datatypes.min.js'));

		// add documentation backend page
		$container['sly-dispatcher']->addListener('SLY_BACKEND_NAVIGATION_INIT', function($nav) use ($container, $name) {
			$config  = $container['sly-config'];
			$disable = $config->get('developer_utils/disable_docs', false);
			$group   = $nav->getGroup('system');

			if (!$disable) {
				if ($group) {
					$service   = $container['wv-documentation-service'];
					$documents = $service->getDocuments(true);
					$page      = $nav->addPage('system', 'devutilsdocs', t('devutils_documentation'));

					if (!empty($documents)) {
						$overview = $page->addSubpage('devutilsdocs', 'Artikeltypen');
						$overview->setExtraParams(array('doc' => ''));

						if ($name === 'devutilsdocs' && !$container['sly-request']->get('doc', 'string')) {
							$overview->forceStatus(true);
						}
					}

					foreach ($documents as $doc => $title) {
						$page->addSubpage('devutilsdocs', $title)->setExtraParams(array('doc' => $doc));
					}
				}
			}

			$disable = $config->get('developer_utils/disable_slice_tooltip', false);

			if (!$disable) {
				$dispatcher = $container['sly-dispatcher'];
				$listener   = array(__CLASS__, 'addDocTooltip');
				$params     = array('container' => $container);

				$dispatcher->addListener('SLY_SLICE_POSTVIEW_ADD',  $listener, $params);
				$dispatcher->addListener('SLY_SLICE_POSTVIEW_EDIT', $listener, $params);
			}

			// add log viewer

			if ($group) {
				$page = $nav->get('system', 'system');
				if ($page) $page->addSubpage('devutilslogviewer', t('devutils_logviewer'));
			}
		});
	}

	public static function addDocTooltip($sliceValues, array $params) {
		$container = $params['container'];
		$service   = $container['sly-service-module'];
		$desc      = $service->get($params['module'], 'description', '');

		if (!empty($desc)) {
			$layout = $container['sly-layout'];
			$script = Util::addOnUri('webvariants/developer-utils', 'js/jquery.tooltip.min.js');

			$layout->addJavaScriptFile($script);

			include DEVUTILS_PATH.'views/tooltip.phtml';
		}
	}

	/**
	 * Wandelt die symbolischen Sprach-Konstanten in echte Clang-IDs
	 *
	 * Diese Hilfsmethode befreit alle anderen Methoden des AddOns von der
	 * lästigen Aufgabe, die als Argument emfpangene (evtl. symbolische)
	 * Sprachkonstante in die aktuelle Sprache umzuformen.
	 *
	 * @param  int $clang  die Sprach-Konstante (>= 0 = echte Clang-ID, sonst symbolische Konstante)
	 * @return int         die echte Sprache, falls WV_Sally::CLANG_CURRENT, null oder false übergeben wurde
	 */
	public static function clang($clang = null) {
		return ($clang === self::CLANG_CURRENT || $clang === false || $clang === null) ? sly_Core::getCurrentClang() : intval($clang);
	}

	public static function clearOutput() {
		while (ob_get_level()) ob_end_clean();
	}

	/**
	 * @return sly_Model_Category
	 */
	public static function getCurrentCategory() {
		$id = self::getCurrentCategoryID();
		return $id >= 0 ? sly_Util_Category::findById($id) : null;
	}

	/**
	 * @return int
	 */
	public static function getCurrentCategoryID() {
		$article = sly_Core::getCurrentArticle();
		return $article ? $article->getCategoryId() : -1;
	}

	public static function isBackend() {
		$lucene = class_exists('WV11') ? WV11::isRunning() : false;
		return sly_Core::isBackend() && !$lucene;
	}

	/**
	 * Ermitteln einer Artikel-ID
	 *
	 * Diese Methode ermittelt zu einem Artikel die dazugehörige ID.
	 *
	 * @param  mixed $article  sly_Model_Article oder int
	 * @return int             die ID oder -1 falls keine gefunden wurde
	 */
	public static function getIDForArticle($article) {
		if (sly_Util_String::isInteger($article))  return (int) $article;
		if ($article instanceof sly_Model_Article) return (int) $article->getId();

		trigger_error(t('devutils_cant_identify_article', $article, gettype($article)), E_USER_WARNING);
		return -1;
	}

	public static function getIDForCategory($category) {
		if (sly_Util_String::isInteger($category))   return (int) $category;
		if ($category instanceof sly_Model_Category) return (int) $category->getId();

		trigger_error(t('devutils_cant_identify_category', $category, gettype($category)), E_USER_WARNING);
		return -1;
	}

	public static function getLoginForUser($user) {
		if (is_string($user))                return (string) $user;
		if ($user instanceof sly_Model_User) return (string) $user->getLogin();

		trigger_error(t('devutils_cant_identify_user', $user, gettype($user)), E_USER_WARNING);
		return null;
	}

	/**
	 * Ermitteln einer Datei-ID
	 *
	 * Diese Methode ermittelt zu einem Dateinamen die dazugehörige ID.
	 *
	 * @param  mixed $medium  der Dateiname (ohne Pfad!) (String) oder die ID (wird als int ungeändert zurückgegeben)
	 * @return int            die ID oder -1 falls keine gefunden wurde
	 */
	public static function getIDForMedium($medium) {
		if (sly_Util_String::isInteger($medium)) return (int) $medium;
		if ($medium instanceof sly_Model_Medium) return (int) $medium->getId();

		if (is_string($medium)) {
			$db = sly_Core::getContainer()->get('sly-persistence');
			$id = $db->magicFetch('file', 'id', array('filename' => $medium));

			if ($id !== false) return (int) $id;
		}

		trigger_error('Konnte Datei-ID für "'.$medium.'" ('.gettype($medium).') nicht ermitteln!', E_USER_WARNING);
		return -1;
	}

	public static function enforceMethod($method) {
		if (strtoupper($method) !== $_SERVER['REQUEST_METHOD']) {
			throw new WV_Exception_Sally('Die Anfrage muss via '.strtoupper($method).' erfolgen.');
		}
	}

	/**
	 * Returns an Article for an Article Identifier
	 *
	 * @param  mixed $articleIdent  Article ID, sly_Model_Article or rex_article
	 * @param  int   $clang         desired target language
	 * @return sly_Model_Article    desired article object
	 */
	public static function getArticle($articleIdent, $clang = null) {
		$id    = self::getIDForArticle($articleIdent);
		$clang = self::clang($clang);

		return sly_Util_Article::findById($id, $clang);
	}

	/**
	 * Gibt die Anzahl aller Artikel einer Kategorie zurück.
	 *
	 * @param  int  $categoryID  die Kategorie
	 * @param  bool $onlineOnly  wenn true, werden nur online Artikel berücksichtigt
	 * @return int               Anzahl von Artikeln in der Kategorie
	 */
	public static function getArticlesCountByCategory($categoryID, $onlineOnly = false) {
		$where = 're_id = ? AND startpage = 1'.($onlineOnly ? ' AND status = 1' : '');

		return WV_SQL::getInstance()->count('article', $where, $categoryID);
	}
}
