<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

class WV8_Listeners {
	protected $container;

	public function __construct(sly_Container $container) {
		$this->container = $container;
	}

	/**
	 * SLY_CACHE_CLEARED
	 */
	public function onClearCache() {
		$app        = $this->container['sly-app'];
		$isSystem   = $app->getCurrentControllerName() === 'system';
		$controller = $app->getCurrentController();
		$clearData  = !$isSystem || $controller->isCacheSelected('wv8-data');
		$rebuild    = !$isSystem || $controller->isCacheSelected('wv8-rebuild');

		if ($clearData) {
			WV8_Util::clearCache();
		}

		if ($rebuild) {
			$this->container['wv8-manager']->rebuild();
		}
	}

	/**
	 * SLY_SYSTEM_CACHES
	 *
	 * @param sly_Form_Select_Base $select
	 */
	public function onSystemCacheList(sly_Form_Select_Base $select) {
		$selected   = $select->getValue();
		$selected[] = 'wv8-data';

		if (sly_Core::isDeveloperMode()) {
			$selected[] = 'wv8-rebuild';
		}

		$select->addValue('wv8-data',    'Global Settings-Cache leeren');
		$select->addValue('wv8-rebuild', 'Global Settings validieren & neu aufbauen');
		$select->setSelected($selected);

		return $select;
	}

	/**
	 * SLY_DB_IMPORTER_AFTER
	 */
	public function onDatabaseImport() {
		WV8_Util::clearCache();

		// and here we go (re-using the implementation of some methods)
		$this->container['wv8-manager']->rebuild();
	}

	/**
	 * SLY_MEDIA_USAGES
	 */
	public function onMediaUsages(array $usages, array $params) {
		$filename = $params['filename'];
		$sql      = WV_SQL::getInstance();
		$provider = $this->container['wv8-provider'];
		$app      = $this->container['sly-app'];
		$prefix   = ht('wv8_title');

		if ($app instanceof sly_App_Backend) {
			$router = $app->getRouter();
		}
		else {
			$router = null;
		}

		// transform filename into a MySQ compatible REGEXP
		$b      = '[^[:alnum:]_+-]'; // more or less like a \b in PCRE
		$quoted = str_replace(array('.', '+'), array('\.', '\+'), $filename);
		$regex  = "(^|$b)$quoted(\$|$b)";

		$sql->query('SELECT `namespace`, `name` FROM ~wv8_values WHERE `values` REGEXP ?', array($regex), '~');

		foreach ($sql as $row) {
			$setting = $provider->getSetting($row['namespace'], $row['name']);
			$params  = array('wv8page' => $setting->getPageID());
			$link    = $router ? $router->getPlainUrl('globalsettings_settings', null, $params) : '#';
			$title   = $prefix.': '.sly_translate($setting->getTitle(), true);

			$usages[] = array('title' => $title, 'link' => $link, 'type' => 'global-setting');
		}

		return $usages;
	}

	/**
	 * SLY_BACKEND_NAVIGATION_INIT
	 */
	public function onBackendNavigation(sly_Layout_Navigation_Backend $nav, array $params) {
		$user = $params['user'];

		if ($user !== null && ($user->isAdmin() || $user->hasPermission('pages', 'globalsettings'))) {
			$group = $nav->getGroup('addon');
			$page  = new sly_Layout_Navigation_Page('globalsettings', t('wv8_title'));

			$this->addSubpages($page, $user);
			$group->addPage($page);
		}
	}

	protected function addSubpages(sly_Layout_Navigation_Page $page, sly_Model_User $user) {
		$manager  = $this->container['wv8-manager'];
		$provider = new WV8_Settings_UserProvider($manager, $user);
		$subpages = $provider->getPages(false);

		// translate & sort the subpages
		$subpages = WV8_Util::translateAndSortList($subpages);

		// add overview first
		$page->addSubpage('globalsettings', t('wv8_overview'));

		// add all subpages to the page
		foreach ($subpages as $pageID => $pagename) {
			$page->addSubpage('globalsettings_settings', $pagename)->setExtraParams(array('wv8page' => $pageID));
		}
	}
}
