<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\metainfo\Property;

use wv\metainfo\Metainfo;
use wv\metainfo\Util;
use wv\metainfo\Property\Property;
use wv\metainfo\Value\Service as ValueService;
use WV_Property;
use sly_DB_PDO_Persistence;

class Synchronizer extends \WV_Service_Property {
	protected $db;
	protected $service;

	public function __construct(sly_DB_PDO_Persistence $db, ValueService $service) {
		$this->db      = $db;
		$this->service = $service;
	}

	public function synchronize(array $oldData, array $newData) {
		$self = $this;

		$this->db->transactional(function() use ($self, $oldData, $newData) {
			foreach (Metainfo::getSupportedTypes() as $type) {
				$oldTypeData = isset($oldData[$type]) ? $oldData[$type] : array();
				$newTypeData = isset($newData[$type]) ? $newData[$type] : array();

				$self->loadFromArray($newTypeData, $oldTypeData, $type);
			}
		});
	}

	public function loadFromArray(array $data, array $oldData = null, $type = null) {
		$this->type = $type;
		return parent::loadFromArray($data, $oldData);
	}

	public function rebuild(array $metainfos) {
		$types  = Metainfo::getSupportedTypes();
		$sql    = $this->db;
		$prefix = $sql->getPrefix();
		$trx    = $sql->beginTrx();

		// remove data for non-existing languages

		$clangs = implode(',', \sly_Util_Language::findAll(true));
		$sql->query('DELETE FROM '.$prefix.'wv2_meta WHERE '.(empty($clangs) ? '1' : 'clang NOT IN ('.$clangs.')'));

		foreach ($types as $type) {
			$list  = isset($metainfos[$type]) ? $metainfos[$type] : array();
			$names = array_keys($list);
			$names = array_map(array($sql, 'quote'), $names);

			// turn all metainfos into real objects

			$this->type = $type;

			foreach ($list as $key => $metainfo) {
				$list[$key] = $this->createProperty($key, $metainfo);
			}

			// delete all metavalues which don't have a valid metainfo

			$sql->query(
				'DELETE FROM '.$prefix.'wv2_meta WHERE meta_type = ?'.
				(empty($names) ? '' : ' AND metainfo NOT IN ('.implode(',', $names).')'),
				array($type)
			);

			// check if the linked objects (articles, categories, files) still exist

			if (!empty($names)) {
				switch ($type) {
					case Metainfo::TYPE_ARTICLE:
						foreach ($list as $name => $metainfo) {
							$aTypes = $metainfo->getArticleTypes();

							// If this metainfo is not assigned to *any* type, delete all occurences
							// of it. Otherwise just delete those metavalues which belong to articles
							// that don't have the required article types.

							if (empty($aTypes)) {
								$articles = '';
							}
							else {
								$aTypes   = array_map(array($sql, 'quote'), $aTypes);
								$articles = ' AND (object_id,clang,revision) NOT IN (SELECT id,clang,revision FROM '.$prefix.'article WHERE `type` IN ('.implode(',', $aTypes).'))';
							}

							$sql->query(
								'DELETE FROM '.$prefix.'wv2_meta WHERE meta_type = ? AND metainfo = ?'.$articles,
								array($type, $name)
							);
						}
						break;

					case Metainfo::TYPE_CATEGORY:
						$sql->query(
							'DELETE FROM '.$prefix.'wv2_meta WHERE meta_type = ? '.
							'AND (object_id,clang,revision) NOT IN (SELECT id,clang,revision FROM '.$prefix.'article WHERE startpage = 1)',
							array($type)
						);
						break;

					case Metainfo::TYPE_MEDIUM:
						$sql->query('DELETE FROM '.$prefix.'wv2_meta WHERE meta_type = ? AND object_id NOT IN (SELECT id FROM '.$prefix.'file)', array($type));
						break;

					case Metainfo::TYPE_USER:
						$sql->query('DELETE FROM '.$prefix.'wv2_meta WHERE meta_type = ? AND object_id NOT IN (SELECT id FROM '.$prefix.'user)', array($type));
				}
			}

			// add missing database rows
			// (doing this after the deletion makes the DELETEs a bit faster in worst
			// case scenarios where just everything is missing and has to be inserted)

			foreach ($list as $metainfo) {
				$this->onNew($metainfo); // run INSERT IGNORE
			}
		}

		$sql->commitTrx($trx);
	}

	protected function createProperty($name, array $property) {
		return Util::createProperty($this->type, $name, $property);
	}

	protected function onChanges(array $oldState, array $newState) {
		Metainfo::clearDataCache();
	}

	protected function onNew(WV_Property $property) {
		$type = $property->getType();
		$name = $property->getName();

		// Be *very* careful about fetching the datatype here. In can happen that
		// the user is just installing a project (with already existing metainfo
		// definitions) and has not yet installed the addOn which provides a
		// certain datatype. If we don't catch this, it becomes impossible to
		// install a project.

		try {
			$default = $property->serialize($property->getDefault());
		}
		catch (\Exception $e) {
			// It's okay to assume NULL here, as the developer is probably going
			// to install a database dump in near future.
			$default = null;

			// scream a bit, maybe it was simply a broken configuration
			trigger_error($e->getMessage(), E_USER_WARNING);
		}

		$sql    = $this->db;
		$prefix = $sql->getPrefix();

		// Standardwert übernehmen
		switch ($type) {
			case Metainfo::TYPE_ARTICLE:

				$arttypes = $property->getArticleTypes();
				if (!empty($arttypes)) {
					$sql->query(
						'INSERT IGNORE INTO '.$prefix.'wv2_meta (object_id,clang,revision,metainfo,meta_type,value) '.
						'SELECT id,clang,revision,?,?,? FROM '.$prefix.'article WHERE type IN (\''.implode('\',\'', $arttypes).'\')',
						array($name, $type, $default)
					);
				}

				break;

			case Metainfo::TYPE_CATEGORY:

				$sql->query(
					'INSERT IGNORE INTO '.$prefix.'wv2_meta (object_id,clang,revision,metainfo,meta_type,value) '.
					'SELECT id,clang,revision,?,?,? FROM '.$prefix.'article WHERE catpos <> 0',
					array($name, $type, $default)
				);

				break;

			case Metainfo::TYPE_MEDIUM:

				$sql->query(
					'INSERT IGNORE INTO '.$prefix.'wv2_meta (object_id,clang,revision,metainfo,meta_type,value) '.
					'SELECT '.$prefix.'file.id,'.$prefix.'clang.id,'.$prefix.'file.revision,?,?,? FROM '.$prefix.'file, '.$prefix.'clang',
					array($name, $type, $default)
				);

				break;

			case Metainfo::TYPE_USER:

				$sql->query(
					'INSERT IGNORE INTO '.$prefix.'wv2_meta (object_id,clang,revision,metainfo,meta_type,value) '.
					'SELECT '.$prefix.'user.id,'.$prefix.'clang.id,'.$prefix.'user.revision,?,?,? FROM '.$prefix.'user, '.$prefix.'clang',
					array($name, $type, $default)
				);
		}
	}

	protected function onDeleted(WV_Property $property) {
		$this->service->deleteProperty($property);
	}

	protected function onChangedDatatype(WV_Property $oldVersion, WV_Property $newVersion) {
		$this->onDeleted($oldVersion); // kill old data
		$this->onNew($newVersion);     // add default value as new data
	}

	protected function onChangedParameters(WV_Property $oldVersion, WV_Property $newVersion) {
		// Wenn sich die Artikeltypen geändert haben, müssen wir noch flugs
		// die Metadaten löschen/hinzufügen, bevor wir ein Update der Daten
		// ausführen.

		$this->onChanged($oldVersion, $newVersion);

		// Jetzt könnnen die bestehenden Daten aktualisiert werden.

		$oldParams = $oldVersion->getParams();
		$newParams = $newVersion->getParams();

		try {
			$actionsToTake = $oldVersion->datatypeCall('getIncompatibilityUpdateStatement', array($oldParams, $newParams));
		}
		catch (Exception $e) {
			return;
		}

		$prefix   = $this->db->getPrefix();
		$metainfo = $oldVersion->getName();

		foreach ($actionsToTake as $action) {
			list ($type, $what, $where) = $action;
			$what  = str_replace('$$$value_column$$$', 'value', $what);
			$where = str_replace('$$$value_column$$$', 'value', $where);
			$where = 'metainfo = '.$this->db->quote($metainfo).' AND meta_type = '.$oldVersion->getType().' AND ('.$where.')';

			switch ($type) {
				case 'DELETE':
					$this->db->query('DELETE FROM '.$prefix.'wv2_meta WHERE '.$where);
					break;

				case 'UPDATE':
					$this->db->query('UPDATE '.$prefix.'wv2_meta SET '.$what.' WHERE '.$where);
					break;

				default:
					trigger_error(t('metainfo_bad_update_statement', $type), E_USER_WARNING);
			}
		}
	}

	protected function onChanged(WV_Property $oldVersion, WV_Property $newVersion) {
		$type = $oldVersion->getType();
		$name = $newVersion->getName();

		if ($type !== Metainfo::TYPE_ARTICLE) {
			return;
		}

		$oldTypes = $oldVersion->getArticleTypes();
		$newTypes = $newVersion->getArticleTypes();

		if ($oldTypes !== $newTypes) {
			$addedTypes   = array_diff($newTypes, $oldTypes);
			$deletedTypes = array_diff($oldTypes, $newTypes);
			$prefix       = $this->db->getPrefix();

			// Daten von den nicht mehr verknüpften Artikeln löschen

			if (!empty($deletedTypes)) {
				$this->db->query('DELETE m FROM '.$prefix.'wv2_meta m, '.$prefix.'article a '.
					'WHERE m.object_id = a.id AND m.metainfo = ? AND m.meta_type = ? '.
					'AND a.type IN (\''.implode('\',\'', $deletedTypes).'\')',
					array($name, $type)
				);
			}

			// Daten zu den neu verknüpften Artikeln hinzufügen

			if (!empty($addedTypes)) {
				$this->db->query(
					'INSERT IGNORE INTO '.$prefix.'wv2_meta '.
					'SELECT id,clang,revision,?,?,? FROM '.$prefix.'article '.
					'WHERE type IN (\''.implode('\',\'', $addedTypes).'\')',
					array($name, $type, $newVersion->getDefault())
				);
			}
		}
	}
}
