<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\metainfo;

use wv\metainfo\Metainfo;

/**
 * Serialisierungsmethoden
 *
 * @ingroup system
 */
class Serializer {
	/**
	 * Artikel-Metadaten speichern
	 *
	 * Diese Funktion speichert die Metainformationen für einen
	 * Artikel, nachdem das Formular abgeschickt wurde.
	 */
	public function processArticleForm($article, $isMeta) {
		$articleType = $article->getType();
		$metainfos   = Metainfo::getAllArticleMetaInfos($articleType, $isMeta);

		return $this->parseFormAndStoreData($article, $metainfos);
	}

	/**
	 * Kategorie-Metadaten speichern
	 *
	 * Diese Funktion speichert die Metainformationen für eine
	 * Kategorie, nachdem das Formular abgeschickt wurde.
	 */
	public function processCategoryForm($category) {
		$metainfos = Metainfo::getAllCategoryMetaInfos();

		return $this->parseFormAndStoreData($category, $metainfos);
	}

	/**
	 * Medium-Metadaten speichern
	 *
	 * Diese Funktion speichert die Metainformationen für ein
	 * Medium, nachdem das Formular abgeschickt wurde.
	 */
	public function processMediumForm($medium) {
		$metainfos = Metainfo::getAllMediumMetaInfos();

		return $this->parseFormAndStoreData($medium, $metainfos);
	}

	/**
	 * Benutzer-Metadaten speichern
	 *
	 * Diese Funktion speichert die Metainformationen für ein
	 * Benutzer, nachdem das Formular abgeschickt wurde.
	 */
	public function processUserForm($user) {
		$metainfos = Metainfo::getAllUserMetaInfos();

		return $this->parseFormAndStoreData($user, $metainfos);
	}

	/**
	 * Benutzer-Metadaten speichern
	 */
	public function processProfileForm($user) {
		$metainfos = Metainfo::getAllUserMetaInfos();

		foreach ($metainfos as $name => $metainfo) {
			if (!$metainfo->getProfile()) {
				unset($metainfos[$name]);
			}
		}

		return $this->parseFormAndStoreData($user, $metainfos);
	}

	protected function parseForm(array $metainfos, &$errors = array()) {
		$values = array();

		// Metainfos durchgehen und parsen
		foreach ($metainfos as $name => $info) {
			/* @var $info \metainfo\Property\Property */
			if ($info->getBackend()) {
				try {
					$inputFromUser = $info->serializeForm();
					$values[$name] = $inputFromUser;
				}
				catch (\WV_Exception_Datatype $e) {
					$errors[$name] = $e->getMessage();
				}
			}
		}

		return $values;
	}

	protected function storeValues($object, array $values) {
		// save values
		$objectType = Metainfo::getTypeForObject($object);
		$container  = Metainfo::getContainer();
		$service    = $container['metainfo-service-value'];

		switch ($objectType) {
			case Metainfo::TYPE_ARTICLE:
			case Metainfo::TYPE_CATEGORY:
				$service->updateObjectData($object->getId(), $object->getClang(), $object->getRevision(), $objectType, $values);
				break;

			case Metainfo::TYPE_MEDIUM:
			case Metainfo::TYPE_USER:
				$languages = \sly_Util_User::getCurrentUser()->getAllowedCLangs();

				foreach ($languages as $language) {
					$newMetadata = array();

					foreach ($values as $metainfo => $value) {
						$newMetadata[$metainfo] = is_array($value) ? $value[$language] : $value;
					}

					$service->updateObjectData($object->getId(), $language, $object->getRevision(), $objectType, $newMetadata);
				}

				break;
		}

		// clear the cache
		Metainfo::clearDataCache();

		// Done!
		return true;
	}

	protected function parseFormAndStoreData($object, array $metainfos) {
		$errors = array();
		$values = $this->parseForm($metainfos, $errors);

		$this->storeValues($object, $values);

		return empty($errors) ? true : $errors;
	}
}
