<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\realurl2\Listener;

use sly_Container;
use sly_Model_Base_Article;
use sly_Service_Article;
use sly_Util_Article;
use wv\realurl2\Cache\Runtime as RuntimeCache;
use wv\realurl2\Generator\Article as ArticleGenerator;
use wv\realurl2\RealURL2;
use wv\realurl2\Util;

/**
 * Sally-Eventlistener
 *
 * Diese Klasse sammelt zu einem Großteil nur Listener, um den URL-Cache immer
 * aktuell zu halten. Für gewöhnlich will niemand diese Methoden direkt
 * aufrufen.
 *
 * @author Christoph
 * @since  forever
 */
class Base {
	protected $container;

	public function __construct(sly_Container $container) {
		$this->container = $container;
	}

	protected function getStorage() {
		return $this->container['realurl2-storage'];
	}

	protected function getRuntimeCache() {
		return $this->container['realurl2-cache-runtime'];
	}

	protected function getService() {
		return $this->container['realurl2-service'];
	}

	protected function getFragment() {
		$fragment = $this->container['sly-request']->post('realurl2_fragment', 'string');

		if ($fragment !== '/') {
			$fragment = rtrim($fragment, '/');
		}

		return $fragment;
	}

	protected function rebuildArticle(sly_Model_Base_Article $article) {
		$path  = $article->getPath();
		$id    = $article->getId();
		$clang = $article->getClang();

		$this->rebuildSingle($id, $clang, $path);
	}

	protected function rebuildSingle($articleID, $clang, $path = null) {
		$runtimeCache = $this->getRuntimeCache();
		$article      = sly_Util_Article::findById($articleID, $clang, sly_Service_Article::FIND_REVISION_BEST);
		$path         = $path === null ? $article->getPath() : $path;
		$revision     = $article->getRevision();

		$runtimeCache->prewarmForArticle($articleID, $clang, $path);

		// get a generator
		$generator = $this->getService()->getGenerator($runtimeCache);
		$url       = $generator->buildUrl($articleID, $clang);

		// store the new URL
		$storage = $this->getStorage();
		$ident   = $articleID.'_'.$clang;

		$storage->setUrl(RealURL2::URL_NAMESPACE, $ident, $url, $articleID, $clang, $revision, $path);
	}

	protected function rebuildList(array $articles, $clang, ArticleGenerator $generator, $storage) {
		$urls = array();

		foreach ($articles as $id => $path) {
			$urls[] = array(
				'ident'      => $id.'_'.$clang,
				'article_id' => $id,
				'clang'      => $clang,
				'revision'   => sly_Util_Article::findById($id, $clang, sly_Service_Article::FIND_REVISION_BEST)->getRevision(),
				'path'       => $path,
				'url'        => $generator->buildUrl($id, $clang)
			);
		}

		// and store them
		$storage->setMany(RealURL2::URL_NAMESPACE, $urls);
	}

	protected function rebuildTree($root, $clang, $path = null, $clear = true) {
		$cache     = $this->getRuntimeCache();
		$storage   = $this->getStorage();
		$generator = $this->getService()->getGenerator($cache);
		$ident     = $root.'_'.$clang;

		if ($path === null) {
			$art  = sly_Util_Article::findById($root, $clang, sly_Service_Article::FIND_REVISION_BEST);
			$path = $art->getPath();
		}

		// remove outdated URLs
		$storage->clearByPath(RealURL2::URL_NAMESPACE, '%|'.$root.'|%', true, $clang);
		$storage->clearByIdent(RealURL2::URL_NAMESPACE, $ident);

		// prewarm
		$tree = Util::getTree($root);
		$cache->prewarmForArticle($root, $clang, $path);
		$cache->prewarmArticleData($clang, array_keys($tree));
		$cache->prewarmForSiteStartArticle($clang);

		// generate the URLs
		$this->rebuildList($tree, $clang, $generator, $storage);
	}
}
