<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\realurl2\Listener;

use sly_Backend_Authorisation_Util;
use sly_Core;
use sly_Form_Select_Base;
use sly_Helper_Message;
use sly_Model_Article;
use sly_Util_Article;
use sly_Util_Language;
use sly_Util_Session;
use wv\realurl2\Cache\Runtime as RuntimeCache;
use wv\realurl2\Dispatcher;
use wv\realurl2\RealURL2;
use wv\realurl2\Resolver;
use wv\realurl2\Util;

/**
 * Sally-Eventlistener
 *
 * Diese Klasse sammelt zu einem Großteil nur Listener, um den URL-Cache immer
 * aktuell zu halten. Für gewöhnlich will niemand diese Methoden direkt
 * aufrufen.
 *
 * @author Christoph
 * @since  forever
 */
class System extends Base {
	/**
	 * WV8_SETTINGS_UPDATED: Global Settings wurden geändert
	 */
	public function settingsUpdated(array $updates) {
		if (!Util::hasSiteStartArticle()) return;

		// test for equality of all settings
		$generated = array();
		$articles  = null;
		$storage   = $this->getStorage();

		foreach ($updates as $settingID => $info) {
			// ignore foreign settings
			if ($info['namespace'] !== 'realurl2') continue;

			if (json_encode($info['old']) !== json_encode($info['new'])) {
				foreach ($info['old'] as $clang => $oldValue) {
					if ($info['new'][$clang] == $oldValue) continue;
					if (in_array($clang, $generated)) continue;

					$generated[] = $clang;

					$cache     = $this->getRuntimeCache();
					$generator = $this->getService()->getGenerator($cache);

					// clear old URLs
					$storage->clearByClang($clang);

					// prewarm
					$cache->prewarmAll($clang);

					// generate & store
					$articles = $articles === null ? Util::getTree(null) : $articles;
					$this->rebuildList($articles, $clang, $generator, $storage);
				}
			}
		}
	}

	/**
	 * SLY_SETTINGS_UPDATED
	 */
	public function slySettingsUpdated($dummy, array $params) {
		$cur = (int) $this->container['sly-config']->get('start_article_id');
		$old = isset($params['originals']) ? $params['originals']['start_article_id'] : 0;

		if ($cur > 0 && $cur !== $old) {
			$this->cacheCleared(true);
		}
	}

	/**
	 * SLY_CACHE_CLEARED: Cache wurde geleert
	 */
	public function cacheCleared($force = false, array $params = array()) {
		if (isset($params['reason']) && $params['reason'] === 'SLY_CLANG_DELETED') {
			return;
		}

		RealURL2::clearCache();

		$isSystem = sly_Core::getCurrentControllerName() === 'system';

		if (!Util::hasSiteStartArticle()) {
			if ($isSystem) {
				print sly_Helper_Message::warn('Der URL-Cache wurde nicht erzeugt, da die Startseite nicht korrekt konfiguriert ist!');
			}

			return;
		}

		// do nothing if requested
		if (!$force && $isSystem) {
			$controller = sly_Core::getCurrentController();

			if (!$controller->isCacheSelected('realurl2')) {
				return;
			}
		}

		// prepare
		$storage   = $this->getStorage();
		$cache     = $this->getRuntimeCache();
		$generator = $this->getService()->getGenerator($cache);
		$articles  = Util::getTree(null);

		// should only clear routes for articles
		$storage->clearByNamespace(RealURL2::URL_NAMESPACE);

		foreach (sly_Util_Language::findAll(true) as $clang) {
			// prewarm
			$cache->prewarmAll($clang);

			// generate & store
			$this->rebuildList($articles, $clang, $generator, $storage);
		}
	}

	/**
	 * OUTPUT_FILTER: Rewriting der Special-URLs aus dem Cache
	 *
	 * Diese Methode läuft über die erzeugte Seite und ersetzt die von Modulen
	 * oder Templates generierten URL-Platzhalter durch die echten URLs.
	 *
	 * Zusätzlich wird diese Methode auch das <base>-Tag im <head> setzen.
	 *
	 * @param  array $params  die von Sally übergebenen Parameter
	 * @return string         der geänderte HTML-Code
	 */
	public function outputFilter($html, array $params) {
		if (RealURL2::doBaseTag()) {
			$base = RealURL2::getBaseTag();

			// <base> hinzufügen oder ändern

			if (strpos($html, '<base>') === false && strpos($html, '<base') === false) {
				$html = preg_replace('#<head( .*?)?>#i', '<head$1>'."\n\t".$base, $html);
			}
			else {
				$html = preg_replace('#<base( .*?)?>#i', $base, $html);
			}
		}

		// replace links (%%realurl2://ARTIKEL-CLANG/?params%%)
		preg_match_all('#%%realurl2://([0-9]+)-([0-9]+)/(.*?)%%#i', $html, $matches, PREG_SET_ORDER);

		$hasUrls = Util::hasSiteStartArticle();

		foreach ($matches as $match) {
			if ($hasUrls) {
				$id     = $match[1];
				$clang  = $match[2];
				$params = $match[3];
				$anchor = '';

				if (mb_strpos($params, '#') !== false) {
					list($params, $anchor) = explode('#', $params, 2);
				}

				// try to restore as much information as possible
				if (!sly_Util_Language::exists($clang)) {
					$clang = sly_Core::getCurrentClang();
				}

				if (!sly_Util_Article::exists($id)) {
					$id = sly_Core::getNotFoundArticleId();
				}

				try {
					$url = RealURL2::getURL($id, $clang, $params);

					if (mb_strlen($anchor) > 0) {
						$url .= '#'.$anchor;
					}
				}
				catch (Exception $e) {
					$url = '#'.$anchor;
				}
			}
			else {
				$url = '#';
			}

			$html = str_replace($match[0], $url, $html);
		}

		// remove broken links
		$html = preg_replace('#%%realurl2:/+[^%]*%+#i', '#', $html);
		$html = preg_replace('#%%realurl2:/+[^"]*"#i', '#"', $html);

		return $html;
	}

	/**
	 * URL_REWRITE: Ersetzen des URL-Rewriting-Algorithmus
	 *
	 * Diese Methode stellt die Integration von realURL2 in Sally dar. Sie wird
	 * für jede vom CMS erzeugte URL aufgerufen und bietet die Möglichkeit,
	 * entsprechend einzugreifen.
	 *
	 * @param  array $params  die von Sally übergebenen Parameter
	 * @return string         die fertige URL für den Artikel
	 */
	public function urlRewrite($url, array $params) {
		$article   = $params['article'];
		$articleID = $article->getId();
		$clang     = $article->getClang();
		$divider   = $params['divider'];
		$isCaching = $params['disable_cache'];
		$params    = $params['params'];

		if ($isCaching) {
			$params = Util::buildQuery($params, $divider);
			return '%%realurl2://'.$articleID.'-'.$clang.'/'.$params.'%%';
		}

		if (!Util::hasSiteStartArticle()) {
			return '#';
		}

		try {
			return RealURL2::getURL($article, $clang, $params, $divider);
		}
		catch (Exception $e) {
			$notFound = sly_Core::getNotFoundArticleId();

			return sly_Util_Article::exists($notFound) ? RealURL2::getURL($notFound, $clang, $params, $divider) : '#';
		}
	}

	/**
	 * SLY_RESOLVE_ARTICLE: Artikel aus dem aktuellen Request ermitteln
	 *
	 * Diese Methode wird direkt nach dem Einbinden des AddOns ausgeführt und
	 * versucht, die aufgerufene URL zu erkennen und die dazugehörigen
	 * GET-Parameter wiederherzustellen.
	 *
	 * Sollte keine URL gefunden werden, so wird der Extension-Point
	 * REALURL2_CLUELESS ausgeführt. Entsprechende Extensions sollten versuchen,
	 * die URL zu erkennen und bei Erfolg auf die von realURL2 erzeugte URL
	 * weiterleiten.
	 *
	 * Nach der Verarbeitung des Requests (auch bei nicht erkannten, solange
	 * die Extensions in REALURL2_CLUELESS nicht wegsterben, um weiterleiten)
	 * wird der Extension-Point REALURL2_URL_DECODED aufgerufen.
	 */
	public function resolveArticle($article) {
		// someone did the job already
		if ($article) return $article;

		$container       = $this->container;
		$request         = $container['sly-request'];
		$checkUserAccess = function($article) use ($container) {
			sly_Util_Session::start(true);

			$user = $container['sly-service-user']->getCurrentUser();

			if (!$user) return false;
			if ($user->isAdmin()) return true;

			// even though the backend app has not been loaded, its classes are
			// available through the autoloader (if we're not unit testing).

			if (!class_exists('sly_Backend_Authorisation_Util')) {
				return false;
			}

			return sly_Backend_Authorisation_Util::canReadArticle($user, $article->getId());
		};

		return $this->container['realurl2-dispatcher']->dispatch($request, $checkUserAccess, false);
	}

	public function articleIsSkipped($dummy, array $params) {
		return Util::isSkipped($params['object']);
	}

	public function articleGetFragment($dummy, array $params) {
		return Util::getFragment($params['object']);
	}

	/**
	 * SLY_SYSTEM_CACHES
	 *
	 * @param sly_Form_Select_Base $select
	 */
	public function systemCacheList(sly_Form_Select_Base $select) {
		$selected   = $select->getValue();
		$selected[] = 'realurl2';

		$select->addValue('realurl2', 'realURL2-Daten neu aufbauen');
		$select->setSelected($selected);

		return $select;
	}
}
