<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace wv\realurl2;

use sly_Core;
use sly_Model_Base_Article;
use sly_Model_Category;
use sly_Service_Article;
use wv\realurl2\Cache\Persistent as Cache;
use wv\realurl2\Cache\Runtime as RuntimeCache;
use wv\realurl2\Generator\Article as Generator;
use WV_SQL;

/**
 * Öffentliche API
 *
 * @author Christoph
 * @since  forever
 */
abstract class RealURL2 {
	protected static $disabledBaseTag = false;  ///< boolean

	const URL_NAMESPACE = 'sally.article';

	/**
	 * URL für einen Artikel ermitteln
	 *
	 * Diese Methode sollte man aufrufen, wenn man für einen Artikel die URL
	 * ermitteln möchte. Sie führt ein entsprechendes Caching durch und wird
	 * automatisch generateURL() aufrufen, sollte die URL noch nicht im Cache
	 * liegen.
	 *
	 * @param  mixed   $article      null für den aktuellen Artikel, ansonsten irgendwas als Artikel identifizierbares
	 * @param  int     $clang        die Sprach-ID
	 * @param  array   $params       GET-Parameter als assoziatives Array
	 * @param  string  $divider      das Trennzeichen für den Query-String
	 * @param  boolean $forceDomain  wenn true, wird immer eine Domain-Angabe ("http://.../") erzeugt (andernfalls nur, wenn nötig)
	 * @return string                die fertige URL für den Artikel (i.d.R. absolut zur Domain)
	 */
	public static function getURL($article = null, $clang = null, $params = array(), $divider = '&amp;', $forceDomain = false) {
		$container = sly_Core::getContainer();
		$clang     = $clang === null ? $container['sly-current-lang-id'] : (int) $clang;

		if ($article instanceof sly_Model_Base_Article) {
			$articleID = $article->getId();
			$revision  = $article instanceof sly_Model_Category ? null : $article->getRevision();

			// For categories, we must force the revision to be null. getRevision() would return
			// a 0 and hence most likely lead to the revision being appended to the final URL
			// (because the start article of that category, which actually controls the URL, is
			// probably not in revision 0 anymore).
		}
		else {
			$articleID = (int) $article;
			$revision  = null;
		}

		$ident = $articleID.'_'.$clang;
		$url   = Cache::findURL(self::URL_NAMESPACE, $ident);

		if ($url === null) {
			$storage = $container['realurl2-storage'];
			$urls    = $storage->findByIdent(self::URL_NAMESPACE, $ident, false, true);

			if (!empty($urls)) {
				$url = reset($urls);
				Cache::cacheURL($url);
			}
		}

		if ($url) {
			return $url->getURL($params, $divider, $forceDomain, $revision);
		}

		// nothing found, generate the URL from scratch
		$service = $container['sly-service-article'];
		$article = $service->findByPK($articleID, $clang, sly_Service_Article::FIND_REVISION_ONLINE);

		if ($article === null) {
			throw new Exception('Invalid article '.$articleID.'/'.$clang.' given!');
		}

		$path = $article->getPath();

		// prewarm a cache
		// Since this should only happen in very rare cases, it's okay to use a
		// non-global cache that will not be used for the next ::getURL() call.
		$runtimeCache = $container['realurl2-cache-runtime'];
		$runtimeCache->prewarmForArticle($articleID, $clang, $article->getPath());

		// get a generator
		$generator = $container['realurl2-service']->getGenerator($runtimeCache);
		$url       = $generator->buildUrl($articleID, $clang);

		// store the new URL
		$storage = $container['realurl2-storage'];
		$url     = $storage->setUrl(self::URL_NAMESPACE, $ident, $url, $articleID, $clang, $path);

		// done!
		return $url->getURL($params, $divider, $forceDomain, $revision);
	}

	/**
	 * Cache leeren
	 *
	 * Diese Methode leert den gesamten Cache.
	 */
	public static function clearCache() {
		Cache::clear(null, null);
	}

	/**
	 * Finds the list of domains that are managed with realURL2.
	 *
	 * @return array  a list with the domains
	 */
	public static function getDomains($addWwwAlternatives = false) {
		$cache = sly_Core::getContainer()->get('sly-cache');
		$key   = 'domain_fragments';
		$data  = $cache->get('realurl2.volatile', $key);

		if ($data === null) {
			$sql       = WV_SQL::getInstance();
			$query     = 'SELECT DISTINCT wv24_fragment FROM ~article WHERE wv24_fragment LIKE "//%"';
			$fragments = $sql->getArray($query, null, '~');

			foreach (Util::getDefaultFragments() as $fragment) {
				$fragments[] = $fragment;
			}

			foreach ($fragments as $fragment) {
				$data[] = Util::getDomainFromFragment($fragment);
			}

			$data = array_unique($data);
			$cache->set('realurl2.volatile', $key, $data);
		}

		if ($addWwwAlternatives) {
			$result = array();

			foreach ($data as $domain) {
				$result[] = $domain;
				$result[] = self::getWwwAlternative($domain);
			}

			$data = $result;
		}

		return $data;
	}

	/**
	 * URL dekodieren
	 *
	 * Diese Methode versucht zu einer gegebenen URL den dazugehörigen Artikel
	 * zu ermitteln. Ggf. wird statt einer Core-URL auch eine API-erzeugte
	 * Route gefunden. Es wird dabei, falls die URL nicht auf einen Slash endet,
	 * auch noch die Variante mit einem angefügten Slash gesucht.
	 *
	 * Der Rückgabewert ist ein assoziatives Array mit allen Feldern aus der
	 * jeweiligen Cache-Tabelle der Datenbank.
	 *
	 * @param  string $url  die zu dekodierende URL
	 * @return URL          null, falls die URL nicht gefunden wurde, sonst eine URL
	 */
	public static function decodeURL($url) {
		return sly_Core::getContainer()->get('realurl2-resolver')->resolveUrl($url);
	}

	/**
	 * @param  boolean $xhtml            true to get self-closing tag, else false
	 * @param  boolean $disableAutoTags  true to disable auto insertion of <base> tag in OUTPUT_FILTER
	 * @return string
	 */
	public static function getBaseTag($xhtml = true, $disableAutoTags = true) {
		if ($disableAutoTags) {
			self::disableAutoBaseTag();
		}

		return sly_Core::getContainer()->get('realurl2-service')->getBaseTag($xhtml);
	}

	public static function enableAutoBaseTag() {
		self::$disabledBaseTag = false;
	}

	public static function disableAutoBaseTag() {
		self::$disabledBaseTag = true;
	}

	/**
	 * @return boolean
	 */
	public static function doBaseTag() {
		return !self::$disabledBaseTag;
	}
}
